/*
 * Copyright 1999-2017 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.polardbx.druid.bvt.sql.mysql.alterTable;

import com.alibaba.polardbx.druid.sql.SQLUtils;
import com.alibaba.polardbx.druid.sql.ast.SQLStatement;
import com.alibaba.polardbx.druid.sql.dialect.mysql.parser.MySqlStatementParser;
import com.alibaba.polardbx.druid.sql.parser.Token;
import junit.framework.TestCase;

public class MySqlAlterTableTest33 extends TestCase {

    public void test_exchange_0() throws Exception {
        String sql = "ALTER TABLE src.`part_tab` EXCHANGE PARTITION p9 WITH TABLE test_create_table without validation";
        MySqlStatementParser parser = new MySqlStatementParser(sql);
        SQLStatement stmt = parser.parseStatementList().get(0);
        parser.match(Token.EOF);

        assertEquals("ALTER TABLE src.`part_tab`\n" +
                "\tEXCHANGE PARTITION p9 WITH TABLE test_create_table WITHOUT VALIDATION", SQLUtils.toMySqlString(stmt));
    }

    public void test_exchange_1() throws Exception {
        String sql = "ALTER TABLE e EXCHANGE PARTITION p0 WITH TABLE e2 WITHOUT VALIDATION;";
        MySqlStatementParser parser = new MySqlStatementParser(sql);
        SQLStatement stmt = parser.parseStatementList().get(0);
        parser.match(Token.EOF);

        assertEquals("ALTER TABLE e\n" +
                "\tEXCHANGE PARTITION p0 WITH TABLE e2 WITHOUT VALIDATION;", SQLUtils.toMySqlString(stmt));
    }

    public void test_exchange_2() throws Exception {
        String sql = "ALTER TABLE e EXCHANGE PARTITION p0 WITH TABLE e2 WITH VALIDATION;";
        MySqlStatementParser parser = new MySqlStatementParser(sql);
        SQLStatement stmt = parser.parseStatementList().get(0);
        parser.match(Token.EOF);

        assertEquals("ALTER TABLE e\n" +
                "\tEXCHANGE PARTITION p0 WITH TABLE e2 WITH VALIDATION;", SQLUtils.toMySqlString(stmt));
    }
}
